//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------
#include <required.glsl>

uniform sampler2D texture;
uniform sc_FragmentPrecision vec2 textureScale;
uniform mat3 textureTransform;

varying vec2 varTex0;

varying vec2 varTex1;
varying vec2 varTex2;
varying vec2 varTex3;
varying vec2 varTex4;

#ifdef VERTEX_SHADER
attribute vec2 position;
attribute vec2 texture0;

void main(void) {
    varTex0 = vec2(textureTransform * vec3(texture0, 1.0));

    varTex1 = varTex0 + (vec2(-1.0, -1.0) * textureScale);
    varTex2 = varTex0 + (vec2(+1.0, -1.0) * textureScale);
    varTex3 = varTex0 + (vec2(-1.0, +1.0) * textureScale);
    varTex4 = varTex0 + (vec2(+1.0, +1.0) * textureScale);

    gl_Position = vec4(position, 0.0, 1.0);
}
#endif

#ifdef FRAGMENT_SHADER
void main(void) {
    // The parameters are hardcoded for now, but could be
    // made into uniforms to control from the program.
    float FXAA_SPAN_MAX = 8.0;
    float FXAA_REDUCE_MUL = 1.0/8.0;
    float FXAA_REDUCE_MIN = 1.0/128.0;
    
    vec3 rgbNW = texture2D(texture, varTex1).rgb;
    vec3 rgbNE = texture2D(texture, varTex2).rgb;
    vec3 rgbSW = texture2D(texture, varTex3).rgb;
    vec3 rgbSE = texture2D(texture, varTex4).rgb;
    vec3 rgbM  = texture2D(texture, varTex0).rgb;
    
    vec3 luma = vec3(0.299, 0.587, 0.114);
    float lumaNW = dot(rgbNW, luma);
    float lumaNE = dot(rgbNE, luma);
    float lumaSW = dot(rgbSW, luma);
    float lumaSE = dot(rgbSE, luma);
    float lumaM  = dot( rgbM, luma);
    
    float lumaMin = min(lumaM, min(min(lumaNW, lumaNE), min(lumaSW, lumaSE)));
    float lumaMax = max(lumaM, max(max(lumaNW, lumaNE), max(lumaSW, lumaSE)));
    
    vec2 dir;
    dir.x = -((lumaNW + lumaNE) - (lumaSW + lumaSE));
    dir.y =  ((lumaNW + lumaSW) - (lumaNE + lumaSE));
    
    float dirReduce = max((lumaNW + lumaNE + lumaSW + lumaSE) * (0.25 * FXAA_REDUCE_MUL), FXAA_REDUCE_MIN);
    
    float rcpDirMin = 1.0/(min(abs(dir.x), abs(dir.y)) + dirReduce);
    
    dir = min(vec2(FXAA_SPAN_MAX,  FXAA_SPAN_MAX),
              max(vec2(-FXAA_SPAN_MAX, -FXAA_SPAN_MAX), dir * rcpDirMin)) * textureScale;
    
    vec4 colorA = 0.5 * ( texture2D(texture, varTex0 - dir * 1.0/6.0) +
                       texture2D(texture, varTex0 + dir * 1.0/6.0));
    vec4 colorB = colorA * 0.5 + 0.25 * (texture2D(texture, varTex0 - dir * 0.5) +
                                     texture2D(texture, varTex0 + dir * 0.5));
    float lumaB = dot(colorB.rgb, luma);
    
    if((lumaB < lumaMin) || (lumaB > lumaMax)) {
        gl_FragColor = colorA;
    } else {
        gl_FragColor = colorB;
    }
}
#endif
