//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------
#include <required.glsl>

//user should define: #define kernelSize [float]
//user should define: #define yFlip [0, 1]
//user should define: #define sigma [float]
//user should define: #define blurStep [float]

uniform sampler2D texture;
uniform vec2 textureScale;

varying vec2 varTex0;


#ifdef VERTEX_SHADER
attribute vec2 position;
attribute vec2 texture0;

void main(void) {
    #if yFlip
        varTex0.x = texture0.x;
        varTex0.y = -texture0.y + 1.0;
    #else
        varTex0 = texture0;
    #endif
    gl_Position = vec4(position, 0.0, 1.0);
}
#endif // VERTEX_SHADER

#ifdef FRAGMENT_SHADER


#ifndef PI
#define PI 3.141592653589793238462643383279
#endif

void main(void) {
    vec4 result = vec4(0.0);

    float radius = floor(kernelSize / 2.0);
    float sigmaTerm = 2.0 * (sigma * sigma);
    float eulerTerm = 1.0 / (sigma * sqrt(2.0 * PI));
    float renormalizationTerm = 0.0;
    
    radius *= blurStep;
    for (float x = -radius; x <= radius; x += blurStep) {
        float gaussianKernelCell = eulerTerm * exp(- (x * x) / sigmaTerm);
        renormalizationTerm += gaussianKernelCell;
        
        vec2 uv = x * textureScale;
        vec4 color = texture2D(texture, varTex0 + uv);

        result += color * gaussianKernelCell;
    }
    
    gl_FragColor = result / renormalizationTerm;
}
#endif // FRAGMENT_SHADER
