//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------
#include <required.glsl>

#ifndef GLES20STD
#define GLES20STD

struct sc_PointLight_t {
    vec3 position;
    vec4 color;
};
struct sc_DirectionalLight_t {
    vec3 direction;
    vec4 color;
};
struct sc_AmbientLight_t {
    vec3 color;
    //vec3 position; // will be used for probes location
    
    vec3 sphericalHarmonics[9];
    
    float intensity;
    float weight;
};
struct sc_AmbientLightCompatibility_t {
    vec3 color;
};
struct sc_Camera_t {
    vec3 position;
    float aspect;
};

uniform vec4 sc_Time;
#define sc_TimeElapsed (sc_Time.x)
#define sc_TimeDelta   (sc_Time.y)

uniform mat4 sc_ModelViewProjectionMatrix;
uniform mat4 sc_ModelViewProjectionMatrixInverse;

uniform mat4 sc_ViewProjectionMatrix;
uniform mat4 sc_ViewProjectionMatrixInverse;

uniform mat4 sc_ModelViewMatrix;
uniform mat4 sc_ModelViewMatrixInverse;

uniform mat4 sc_ProjectionMatrix;
uniform mat4 sc_ProjectionMatrixInverse;

uniform mat4 sc_ViewMatrix;
uniform mat4 sc_ViewMatrixInverse;

uniform mat4 sc_ModelMatrix;
uniform mat4 sc_ModelMatrixInverse;

uniform mat3 sc_NormalMatrix;
uniform mat3 sc_NormalMatrixInverse;

uniform mat3 sc_ViewNormalMatrix;
uniform mat3 sc_ViewNormalMatrixInverse;

uniform sc_Camera_t sc_Camera;
uniform sampler2D sc_ScreenTexture;

#ifdef sc_PointLightsCount
uniform sc_PointLight_t sc_PointLights[sc_PointLightsCount];
#endif

#ifdef sc_DirectionalLightsCount
uniform sc_DirectionalLight_t sc_DirectionalLights[sc_DirectionalLightsCount];
#endif

#ifdef sc_AmbientLightsCount
uniform sc_AmbientLight_t sc_AmbientLights[sc_AmbientLightsCount];
#endif

// backward compatibility
sc_AmbientLightCompatibility_t getAmbientLightCompatibility() {
    sc_AmbientLightCompatibility_t ambient;
#ifdef sc_AmbientLightsCount
    ambient.color = sc_AmbientLights[0].color * sc_AmbientLights[0].intensity;
#else
    ambient.color = vec3(0.0);
#endif
    return ambient;
}
#define sc_AmbientLight (getAmbientLightCompatibility())

#ifdef sc_ProjectiveShadowsCaster
uniform float sc_ShadowDensity;

vec4 getShadowColor(float alpha) {
    float shadowTerm = 1.0 - sc_ShadowDensity * alpha;
    return vec4(shadowTerm, shadowTerm, shadowTerm, 1.0);
}
#endif // sc_ProjectiveShadowsCaster

#ifdef sc_ProjectiveShadowsReceiver
varying vec2 varShadowTex;

uniform sampler2D sc_ShadowTexture;
uniform mat4 sc_ProjectorMatrix;

vec2 getProjectedTexCoords(vec4 vertexPosition) {
#if (sc_RenderingSpace == 2) //SC_RENDERING_SPACE_MODEL
    vec4 worldPos = vertexPosition;
#elif (sc_RenderingSpace == 1) //SC_RENDERING_SPACE_OBJECT
    vec4 worldPos = sc_ModelMatrix * vertexPosition;
#else
#error Unsupported sc_RenderingSpace in getProjectedTexCoords()!
#endif
    vec4 projectedUVs = sc_ProjectorMatrix * worldPos;
    return projectedUVs.xy / projectedUVs.w * 0.5 + 0.5;
}

float getShadowSample() {
    float result = 1.0;
    if (varShadowTex.x > 0.0 && varShadowTex.x < 1.0 && varShadowTex.y > 0.0 && varShadowTex.y < 1.0) {
        result = texture2D(sc_ShadowTexture, varShadowTex).r;
    }
    return result;
}
#endif // sc_ProjectiveShadowsReceiver

#endif //GLES20STD
