//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------

#include <std.glsl>

uniform mat4 pointsModelInvMatrix[MAX_LIQUIFY];      // points modelInv matrix
uniform mat4 pointsModelViewProjMatrix[MAX_LIQUIFY]; // points modelviewprojInv matrix
uniform vec4 spcoeffs[MAX_LIQUIFY];                  // ptx,pty, coef,rad

varying vec4 varScreenSpacePointsPos[MAX_LIQUIFY]; // xyzw positions in post perspective space
varying vec2 varScreenPos;

//#define DEBUG_RATIO
//#define DEBUG_COVERAGE

#ifdef VERTEX_SHADER
attribute vec2 atbCoord2d;

void main(void) {
    varScreenPos = atbCoord2d;
    gl_Position = vec4(atbCoord2d.x, atbCoord2d.y, 0.0, 1.0);

    vec4 screenPos = vec4(atbCoord2d.x, atbCoord2d.y, -1.0, 1.0);

    // compute vertex location in world space
    vec4 worldPos4 = sc_ViewProjectionMatrixInverse * screenPos;
    vec3 worldPos = worldPos4.xyz / worldPos4.w;

    for (int i = 0; i < MAX_LIQUIFY; i++) {
        // compute vertex in object space
        vec4 vertexPosObjSpace = pointsModelInvMatrix[i] * vec4(worldPos, 1.0);
        vec4 camPosObjSpace = pointsModelInvMatrix[i] * vec4(sc_Camera.position.xyz, 1.0);
        vec3 viewDirObjSpace = normalize(vertexPosObjSpace.xyz - camPosObjSpace.xyz);

        // float denom = normal.dot(ray.direction);
        float denom = viewDirObjSpace.z;
        float t = 10000.0;
        if (denom != 0.0) // should never happened (clip from CPU code)
        {
            // float t = (center - ray.origin).dot(normal) / denom;
            t = -camPosObjSpace.z / denom;
        }
        if (t < 0.0)
            t = 10000.0;
        // intersection in point object space
        vec3 ptObjSpace = camPosObjSpace.xyz + t * viewDirObjSpace;
        //
        vec4 ptScreenPos = pointsModelViewProjMatrix[i] * vec4(ptObjSpace.x, ptObjSpace.y, 0.0, 1.0);
        varScreenSpacePointsPos[i] = vec4(ptObjSpace.x, ptObjSpace.y, ptScreenPos.z, ptScreenPos.w);
    }
}
#endif

#ifdef FRAGMENT_SHADER

vec2 liquifyVector(vec2 initialVector, float curDistance, float radius, float coeficient) {
    float interpolationFactor = curDistance / radius;
    float newR = pow(interpolationFactor, coeficient) * radius;
    vec2 newVector = initialVector * (newR / curDistance);
    return newVector;
}

void main(void) {
    vec2 fromPointCenterVector;
    vec2 liqVec = vec2(0.0, 0.0);
    for (int i = 0; i < MAX_LIQUIFY; ++i) {
        float dist = length(varScreenSpacePointsPos[i].xy);
        if (dist < spcoeffs[i].w ) {
            fromPointCenterVector = varScreenPos.xy - spcoeffs[i].xy;
            liqVec = liqVec + liquifyVector(fromPointCenterVector, dist, spcoeffs[i].w, spcoeffs[i].z) -
                     fromPointCenterVector;
        }
    }

    vec2 posToFetch = varScreenPos.xy + liqVec.xy;
    gl_FragColor = texture2D(sc_ScreenTexture, posToFetch * 0.5 + 0.5);
#ifdef DEBUG_RATIO
    float offset = length(liqVec) * 10.0;
    gl_FragColor = vec4(offset, offset, offset, 1.0);
#endif
#ifdef DEBUG_COVERAGE
    gl_FragColor = vec4(1.0, 0.0, 0.0, 1.0);
#endif
}

#endif
