//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------

uniform sampler2D sc_ScreenTexture;


uniform vec4 uniNetStep;
uniform float uniWidth;
uniform float uniHeight;
uniform float uniRadius;
uniform sampler2D uniShift;
uniform vec4 uniStep;
varying vec2 coord;

//#define DEBUGSHIFTS

#ifdef VERTEX_SHADER
attribute vec2 atbCoord2d;

void main(void)
{
    coord = 0.5 * atbCoord2d + 0.5;
    gl_Position = vec4(atbCoord2d.x,-atbCoord2d.y, 0.0, 1.0);
}

#endif

#ifdef FRAGMENT_SHADER

float maxRange = 0.5;

vec2 makeFloats(vec4 read)
{
    return read.xz*read.yw*maxRange-maxRange*0.5;
}

void main(void)
{
    float dx = mod(coord.x, uniStep.x);
    float dy = mod(coord.y, uniStep.y);
    vec2 from = vec2(coord.x - dx, coord.y - dy);
    dx /= uniStep.x;
    dy /= uniStep.y;
    
    vec4 read = texture2D(uniShift, from);
    vec4 nxPosition = read * ((1.0 - dx) * (1.0 - dy));
    
    read = texture2D(uniShift, from + vec2(uniStep.x, 0.0));
    nxPosition += read * (dx * (1.0 - dy));
    
    read = texture2D(uniShift, from + vec2(0.0, uniStep.y));
    nxPosition += read * ((1.0 - dx) * dy);
    
    read = texture2D(uniShift, from + vec2(uniStep.x, uniStep.y));
    nxPosition += read * (dx * dy);
    
    vec2 position = makeFloats(nxPosition);
    
    position += coord;
    
    position.y = 1.0-position.y;
    
    gl_FragColor = texture2D(sc_ScreenTexture, position);

#ifdef DEBUGSHIFTS
    gl_FragColor = vec4((position.x-coord.x)*5.0+0.5,(position.y-coord.y)*5.0+0.5,0.0,1.0);
#endif
}

#endif
