//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------

uniform sampler2D sc_ScreenTexture;

varying vec2 coord;

uniform vec4 uniNetStep;
uniform float uniWidth;
uniform float uniHeight;
uniform float uniRadius;
uniform sampler2D uniShift;
#ifdef DEBUGMODE
uniform sampler2D debugTex;
#endif

// uncomment to debug shifts in color
//#define DEBUGSHIFTS

#ifdef VERTEX_SHADER
attribute vec2 atbCoord2d;

#ifdef CHARTEX
vec2 makeFloats(vec4 read)
{
    return 255.0 * (read.xz * 16.0 + read.yw / 16.0)  - 2048.0;
}
#endif


void main(void)
{
    float stepX = float(uniNetStep.x) / float(uniWidth);
    float stepY = float(uniNetStep.y) / float(uniHeight);
    float x = atbCoord2d.x * 0.5 + 0.5;
    float y = atbCoord2d.y * 0.5 + 0.5;
    //
#ifdef CHARTEX
    vec4 sum4 = texture2D(uniShift, vec2(x-0.5 * stepX, y-0.5 * stepY));
    vec2 floatSum = makeFloats(sum4);
#else
    vec2 floatSum = texture2D(uniShift, vec2(x-0.5 * stepX, y-0.5 * stepY)).xy;
#endif
    
    coord = vec2(x,y)+floatSum.xy/vec2(uniWidth,uniHeight);
    coord.y = 1.0-coord.y;

    gl_Position = vec4(atbCoord2d.x,-atbCoord2d.y, 0.0, 1.0);

#ifdef DEBUGSHIFTS
    coord = floatSum/vec2(uniWidth,uniHeight);
#endif
}

#endif

#ifdef FRAGMENT_SHADER

void main(void)
{
    #ifdef DEBUGMODE
        gl_FragColor = texture2D(debugTex, coord);
    #else
        gl_FragColor = texture2D(sc_ScreenTexture, coord);
    #endif
#ifdef DEBUGSHIFTS
    gl_FragColor = vec4(coord.x*5.0+0.5,coord.y*5.0+0.5,0.0,1.0);
#endif
}

#endif
