//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------

#include <std.glsl>
#include <std_vs.glsl>
#include <std_fs.glsl>
#include "includes/utils.glsl"


void sc_VSMain(inout sc_Vertex_t vertex) {}


#ifdef FRAGMENT_SHADER


uniform sampler2D texSampler;
uniform sampler2D texSampler2;
uniform sampler2D texSamplerSh;

uniform int uniNumSamples;
uniform int uniRandMod;
uniform int uniRandSeed;
uniform float uniRoughness;
uniform bool uniRenderMip;
uniform int uniMipToRender;
uniform vec2 uniTopMipRes;
uniform int uniMaxRoughnessMip;
uniform float uniRoughnessExponentInv;
uniform bool uniRenderIrradiance;
uniform bool uniRenderSh;
uniform bool uniHDR;

#include "includes/envmap.glsl"

vec3 sampleRadiance(vec3 R) {
    vec2 uv = calcPanoramicTexCoordsFromDir(R, 0.0);
    if (uniHDR) {
        uv.y = 1.0 - uv.y; // need to undo the flip at bake time.
    }
    vec4 color = texture2DLod(texSampler, uv, 0);
    if (uniHDR) {
        return decodeRGBE(color);
    } else {
        return srgbToLinear(color.rgb);
    }
}

vec3 sampleSpecularEnvmapLod(vec3 R, float lod) {
    vec2 uv = calcPanoramicTexCoordsFromDir(R, 0.0);
    uv.y = 1.0 - uv.y; // need to undo the flip at bake time.
    
    float lodFloor = floor(lod);
    float lodCeil = ceil(lod);
    float lodFrac = lod - lodFloor;
    
    vec2 uvFloor = calcSeamlessPanoramicUvsForSampling(uv, uniTopMipRes, int(lodFloor));
    vec4 texFloor = texture2DLod(texSampler2, uvFloor, lodFloor);
    
    vec2 uvCeil = calcSeamlessPanoramicUvsForSampling(uv, uniTopMipRes, int(lodCeil));
    vec4 texCeil = texture2DLod(texSampler2, uvCeil, lodCeil);
    
    vec4 tex = mix(texFloor, texCeil, lodFrac);
    return decodeRGBD(tex);
}

vec3 calculateDiffuseIrradiance(vec3 V) {
    return vec3(0.0); // Dummy
}

#include "includes/pbr.glsl"
#include "includes/spherical_harmonics.glsl"

void main() {
    vec4 fragColor = vec4(0.0);
    
    vec2 texCoords = varScreenTexturePos;
    
    const float ShCompressionFactor = 3;

    DebugOptions debug = defaultDebugOptions();
    debug.envSampling = true;
    debug.envSamples = uniNumSamples;
    debug.envRandMod = uniRandMod;
    debug.envRandSeed = uniRandSeed;
    debug.envBentNormal = false;

    if (uniRenderMip) {
        // Render convolved environment map mips.
        vec2 uv = calcSeamlessPanoramicUvsForConvolution(texCoords, uniTopMipRes, uniMipToRender);
        vec3 R = calcDirFromPanoramicTexCoords(uv);
        vec3 texColor = vec3(0.0);
        if (uniMipToRender == 0) {
            texColor = sampleRadiance(R);
        } else {
            texColor = prefilterEnvmap(uniRoughness, R, debug);
        }
        fragColor = encodeRGBD(texColor); // We use RGBD encoding, because RGBE exhibits banding under bilinear filtering.
    } else if (uniRenderIrradiance) {
        // Render an irradiance env map.
        vec2 uv = calcSeamlessPanoramicUvsForConvolution(texCoords, uniTopMipRes, uniMipToRender);
        vec3 R = calcDirFromPanoramicTexCoords(uv);
        vec3 texColor = prefilterEnvmap(1.0, R, debug);
        fragColor = encodeRGBD(texColor); // We use RGBD encoding, because RGBE exhibits banding under bilinear filtering.
    } else if (uniRenderSh) {
        // Render 9 spherical harmonics coefficients per rgb channel into a 1x9 render target.
        int pixel = int(texCoords.x * NumShCoeffs);
        vec3 coeff = projectEnvmapToShCoefficient(pixel, int(uniTopMipRes.x));
        coeff = coeff / ShCompressionFactor + 0.5;
        fragColor = vec4(coeff, 1);
    }
    
    gl_FragColor = fragColor;
}

#endif // FRAGMENT_SHADER
