#include <std.glsl>
#include <std_vs.glsl>
#include <std_fs.glsl>

uniform vec4 mainColor;

#if !defined(NOTEXTURE)
uniform sampler2D mainTexture;
uniform mat3      mainTextureTransform;
#endif

uniform sampler2D normalmap;

// Cook-Torrance parameters
uniform vec3 specular;
uniform float roughness;

void sc_VSMain(inout sc_Vertex_t vertex) {
#if !defined(NOTEXTURE)
    vertex.texture0 = vec2(mainTextureTransform * vec3(vertex.texture0, 1.0));
#endif
}

#ifdef FRAGMENT_SHADER
#define G_COOK_TORRANCE
#define D_BECKMANN
#include "includes/brdf/microfacet/brdf.glsl"

void main(void) {
#if defined(NOTEXTURE)
    vec4 albedo = mainColor;
#else
    vec4 albedo = texture2D(mainTexture, varTex0) * mainColor;
#endif
    
    vec3 result = albedo.rgb * sc_AmbientLight.color;
    
#ifdef NORMALMAP
    vec3 N = unpackNormal(normalmap, varTex0);
#else
    vec3 N = normalize(varNormal);
#endif
    vec3 V = normalize(sc_Camera.position - varPos);
    
#if defined sc_PointLightsCount
    for(int i = 0; i < sc_PointLightsCount; ++i) {
        sc_PointLight_t light = sc_PointLights[i];
        vec3 L = normalize(light.position - varPos);
        result += brdf(N, L, V, roughness, albedo.rgb, specular, light.color.rgb);
    }
#endif
    
#if defined sc_DirectionalLightsCount
    for(int i = 0; i < sc_DirectionalLightsCount; ++i) {
        sc_DirectionalLight_t light = sc_DirectionalLights[i];
        vec3 L = normalize(light.direction);
        result += brdf(N, L, V, roughness, albedo.rgb, specular, light.color.rgb);
    }
#endif
    
    gl_FragColor = vec4(result, albedo.a);
}
#endif
