#include <std.glsl>
#include <std_vs.glsl>
#include <std_fs.glsl>

#ifndef ENABLE_CUSTOM_BLEND
	#define ENABLE_CUSTOM_BLEND
#endif

#define BLEND_MODE_SCREEN

uniform vec4      mainColor;

uniform sampler2D mainTexture;

#if !defined(NOMASK)
uniform sampler2D maskTexture;
#endif

void sc_VSMain(inout sc_Vertex_t vertex) {}

#include "includes/blend_modes.glsl"

#ifdef FRAGMENT_SHADER

vec3 grayscale(vec3 color)
{
    return vec3(0.2126 * color.r + 0.7152 * color.g + 0.0722 * color.b);
}

vec3 gammaCorrection(vec3 color, float gamma)
{
    return pow(color, vec3(1.0/gamma));
}

vec3 levelsControlInputRange(vec3 color, float minInput, float maxInput)
{
    vec3 range = max(vec3(0.0), color - vec3(minInput));
    return min(range / vec3(maxInput - minInput), vec3(1.0));
}

vec3 levelsControlInput(vec3 color, float minInput, float gamma, float maxInput)
{
    vec3 inputRange = levelsControlInputRange(color, minInput, maxInput);
    return gammaCorrection(inputRange, gamma);
}

vec3 levelsControlOutputRange(vec3 color, float minOutput, float maxOutput)
{
    return mix(vec3(minOutput), vec3(maxOutput), color);
}

vec3 levelsControl(vec3 color,
                   float minInput, float gamma, float maxInput,
                   float minOutput, float maxOutput)
{
    color = levelsControlInput(color, minInput, gamma, maxInput);
    return levelsControlOutputRange(color, minOutput, maxOutput);
}

void main(void)
{
    vec3 a = getFramebufferColor().rgb;

    vec4 b = texture2D(mainTexture, varTex0) * mainColor;

    vec3 c = grayscale(a);
    vec3 d = levelsControl(c, 0.0/255.0,  3.0, 211.0/255.0, 0.0, 1.0);
    vec3 e = b.rgb * d;
    vec3 f = levelsControl(c, 0.0/255.0,  0.14, 255.0/255.0, 0.0, 1.0);
    vec3 j = BlendScreen(f, e);
    
    float maskAlpha;
    #if defined(NOMASK)
		maskAlpha = 1.0;
    #else
        maskAlpha = texture2D(maskTexture, varTex1).r;
    #endif

    gl_FragColor = vec4(j, b.a * maskAlpha);
}

#endif // FRAGMENT_SHADER
