#include <std.glsl>
#include <std_vs.glsl>
#include <std_fs.glsl>

uniform float radius;
uniform float sigma;

#define STEP_COUNT 7

#ifdef VERTICAL
	#define BLUR_DIRECTION 1
#else
	#define BLUR_DIRECTION 0
#endif

#ifndef PI
	#define PI 3.141592
#endif

varying vec2 coords;

#ifdef VERTEX_SHADER

void sc_VSMain(inout sc_Vertex_t vertex) {
    vec4 screenPosition = sc_ObjectToScreen(vertex.position);
    coords = screenPosition.xy / screenPosition.w * 0.5 + 0.5;
}

#endif //VERTEX_SHADER



#ifdef FRAGMENT_SHADER

float getGaussianCoefficient(float x) {
    return (1.0 / sqrt((2.0 * PI * sigma * sigma))) * exp(-((x * x) / (2.0 * sigma * sigma)) );
}

void main()
{
    vec2 swapOrient = vec2(1.0 - float(BLUR_DIRECTION), float(BLUR_DIRECTION));

	float blurRadius = radius / 100.0;
	float blurStep = blurRadius * 2.0 / float(STEP_COUNT);

	vec2 startPos = coords - vec2(blurRadius, blurRadius) * swapOrient;

    vec4 blurredColor = vec4(0.0, 0.0, 0.0, 1.0);
    float Z = 0.0;

    for (int i = 0; i < int(STEP_COUNT); i++) {
		float coeff = getGaussianCoefficient(float(STEP_COUNT) * (-1.0) / 2.0 + float(i));

		vec2 pixCoords = startPos + vec2(blurStep * float(i)) * swapOrient;
	    blurredColor += texture2D(sc_ScreenTexture, pixCoords) * coeff;

	    Z += coeff;
    }

    gl_FragColor = blurredColor / Z;
}

#endif //FRAGMENT_SHADER
