//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------

#include <std.glsl>
#include <std_vs.glsl>
#include <std_fs.glsl>

uniform vec4 mainColor;

uniform sampler2D mainTexture;
#if !defined(NOMASK)
uniform sampler2D maskTexture;
#endif


void sc_VSMain(inout sc_Vertex_t vertex) {
}


#ifdef FRAGMENT_SHADER

vec4 mapColor(vec4 orgColor)
{
    const float EPS = 0.000001;
    const float pxSize = 512.0;
    
    float bValue = (orgColor.b * 255.0) / 4.0;
    
    vec2 mulB = clamp(floor(bValue) + vec2(0.0, 1.0), 0.0, 63.0);
    vec2 row = floor(mulB / 8.0 + EPS);
    vec4 row_col = vec4(row, mulB - row * 8.0);
    const vec4 offset = vec4(1.0 - 0.5/pxSize, 1.0 - 0.5/pxSize, -0.5/pxSize, -0.5/pxSize);
    vec4 lookup = orgColor.ggrr * (63.0/pxSize) + row_col * (64.0/pxSize) + (0.5/pxSize);
    float b1w = bValue - mulB.x;
    
    vec3 sampled1 = texture2D(mainTexture, vec2(lookup.z, 1.0 - lookup.x)).rgb;
    vec3 sampled2 = texture2D(mainTexture, vec2(lookup.w, 1.0 - lookup.y)).rgb;
    
    vec3 res = mix(sampled1, sampled2, b1w);
    return vec4(res, orgColor.a);
}

void main(void) {
    vec4 originalColor = getFramebufferColor();
    vec4 color = mapColor(originalColor) * mainColor;
#if defined(NOMASK)
    float factor = 1.0;
#else
    float factor = texture2D(maskTexture, varTex1).r;
#endif

    gl_FragColor = vec4(mix(originalColor.rgb, color.rgb, factor * color.a), 1.0);
}

#endif
