#include <std.glsl>
#include <std_vs.glsl>
#include <std_fs.glsl>

uniform vec2  uniCenter;
uniform vec2  uniMinMaxBlurEdge;
uniform float uniSize;


// SIZE_COEFF is coefficient for decreasing blur size for lowEnd devices with less samples

#if SC_DEVICE_CLASS == SC_DEVICE_CLASS_A
	#define BLUR_SAMPLE_COUNT 4
	#define SIZE_COEFF 0.7
#elif SC_DEVICE_CLASS == SC_DEVICE_CLASS_B
	#define BLUR_SAMPLE_COUNT 6
	#define SIZE_COEFF 0.8
#elif SC_DEVICE_CLASS == SC_DEVICE_CLASS_C
	#define BLUR_SAMPLE_COUNT 8
	#define SIZE_COEFF 0.9
#elif SC_DEVICE_CLASS == SC_DEVICE_CLASS_D
	#define BLUR_SAMPLE_COUNT 10
	#define SIZE_COEFF 1.0
#else
	#define BLUR_SAMPLE_COUNT 4
	#define SIZE_COEFF 0.7
#endif


//Increase for better blur and worse performance
#define BLUR_SAMPLE_COEFF 1.0

varying vec2 coords;

#ifdef VERTEX_SHADER

void sc_VSMain(inout sc_Vertex_t vertex) {
    vec4 screenPosition = sc_ObjectToScreen(vertex.position);
    coords = screenPosition.xy / screenPosition.w * 0.5 + 0.5;
}

#endif //VERTEX_SHADER



#ifdef FRAGMENT_SHADER

vec3 blur(vec2 uv, float uniSize) {
	int blurSamples = int(BLUR_SAMPLE_COEFF * float(BLUR_SAMPLE_COUNT));

    float invSamples = 1.0 / float(blurSamples);
    vec2 dir = normalize(uv - uniCenter);
    vec2 p0 = uv - dir * uniSize * SIZE_COEFF;
    vec2 p1 = uv + dir * uniSize * SIZE_COEFF;
    vec2 step = (p1 - p0) * invSamples;
    vec3 col = vec3(0.0);
    vec3 col0 = texture2D(sc_ScreenTexture, uv).rgb;
    float mid = 0.5 * float(blurSamples);
    for(int i = 0; i < blurSamples; i++) {
        float mx = abs(mid - float(i)) / mid;
        mx = pow(1.09, mx);
        col += col0 * (1.0 - mx) + texture2D(sc_ScreenTexture, p0).rgb * mx;
        p0 += step;
    }
    return col * invSamples;
}

void main() {
    float dist = length(coords - uniCenter);

    float lerpFactor = smoothstep(uniMinMaxBlurEdge.x, uniMinMaxBlurEdge.y, dist);

    gl_FragColor.rgb = mix(texture2D(sc_ScreenTexture, coords).rgb, blur(coords, uniSize / 100.0), lerpFactor);
    gl_FragColor.a = 1.0;
}

#endif //FRAGMENT_SHADER
