//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------

#include <std.glsl>
#include <std_vs.glsl>
#include <std_fs.glsl>

uniform sampler2D mainTexture;
uniform float intensityRadius;

varying vec2 texCoord0;
varying vec2 texCoord1;
varying vec2 texCoord2;
varying vec2 texCoord3;
varying vec2 texCoord4;


#ifdef VERTEX_SHADER

void sc_VSMain(inout sc_Vertex_t vertex) {
    vec4 screenPosition = sc_ObjectToScreen(vertex.position);

    texCoord0 = screenPosition.xy / screenPosition.w * 0.5 + 0.5;
    texCoord1 = texCoord0 + vec2(-0.00694444, -0.00329861);
    texCoord2 = texCoord0 + vec2(-0.00694444,  0.00607639);
    texCoord3 = texCoord0 + vec2( 0.00972222, -0.00329861);
    texCoord4 = texCoord0 + vec2( 0.00972222,  0.00607639);
}

#endif //VERTEX_SHADER



#ifdef FRAGMENT_SHADER

float getLuminance(vec4 color) {
    return 0.299 * color.r + 0.587 * color.g + 0.114 * color.b;
}
float rand(vec2 co) {
    return fract(sin(dot(co.xy, vec2(12.9898,78.233))) * 43758.5453);
}
float getWeight(vec2 intensity) {
    float lg = log(intensity.x / intensity.y);
    lg *= lg;
    return exp(-lg / (2.0 * intensityRadius * intensityRadius));
}

void main(void) {
     vec4 nextColor;
     float sum = 1.0;
     vec4 screenColor = texture2D(sc_ScreenTexture, texCoord0);
     gl_FragColor = screenColor;
     float intens = getLuminance(gl_FragColor);
     float nextIntens, curr;

     nextColor = texture2D(sc_ScreenTexture, texCoord1);
     nextIntens = getLuminance(nextColor);
     curr = 0.36787944 *  getWeight(vec2(intens, nextIntens));
     sum += curr;
     gl_FragColor += curr * nextColor;

     nextColor = texture2D(sc_ScreenTexture, texCoord2);
     nextIntens = getLuminance(nextColor);
     curr = 0.36787944 *  getWeight(vec2(intens, nextIntens));
     sum += curr;
     gl_FragColor += curr * nextColor;

     nextColor = texture2D(sc_ScreenTexture, texCoord3);
     nextIntens = getLuminance(nextColor);
     curr = 0.36787944 *  getWeight(vec2(intens, nextIntens));
     sum += curr;
     gl_FragColor += curr * nextColor;

     nextColor = texture2D(sc_ScreenTexture, texCoord4);
     nextIntens = getLuminance(nextColor);
     curr = 0.36787944 *  getWeight(vec2(intens, nextIntens));
     sum += curr;
     gl_FragColor += curr * nextColor;
     
 #ifdef GL_FRAGMENT_PRECISION_HIGH
     float noise = (rand(texCoord0) - 0.5) / 30.0;
     gl_FragColor = gl_FragColor / sum + vec4(noise, noise, noise, 1.0);
 #else
     gl_FragColor = gl_FragColor / sum;
 #endif

    float factor =  texture2D(mainTexture, varTex0).r;
    gl_FragColor = mix(screenColor, gl_FragColor, factor);
 }

#endif //FRAGMENT SHADER
