#include <std.glsl>
#include <std_vs.glsl>
#include <std_fs.glsl>

uniform float chromaticAberration;
uniform float baseSaturation;

uniform float bandHeight;
uniform float bandAmplitude;
uniform float bandSpeed;
uniform float bandInterval;

varying vec2 coords;

#ifdef VERTEX_SHADER

void sc_VSMain(inout sc_Vertex_t vertex) {
    vec4 screenPosition = sc_ObjectToScreen(vertex.position);
    coords = screenPosition.xy / screenPosition.w * 0.5 + 0.5;
}

#endif //VERTEX_SHADER



#ifdef FRAGMENT_SHADER

vec3 RGB2YUV(vec3 RGB)
{
    // RGB to YUV
    float Wr = 0.299;
    float Wg = 0.587;
    float Wb = 0.114;
    float Umax = 0.436;
    float Vmax = 0.615;

    float R = RGB.r;
    float G = RGB.g;
    float B = RGB.b;

    float Y = Wr * R + Wg * G + Wb * B;
    float U = Umax * (B - Y) / (1.0 - Wb);
    float V = Vmax * (R - Y) / (1.0 - Wr);

    return vec3(Y, U, V);
}

vec3 YUV2RGB(vec3 YUV)
{
    float R = YUV.x + 1.140 * YUV.z;
    float G = YUV.x - 0.395 * YUV.y - 0.581 * YUV.z;
    float B = YUV.x + 2.033 * YUV.y;

    return vec3(R, G, B);
}

void main()
{
    vec2 warpUV = coords;

    float warpBandY = bandHeight + mod(sc_TimeElapsed * -bandSpeed, bandInterval);

    float warpAmount = smoothstep(warpBandY, warpBandY - bandHeight, warpUV.y);

    warpUV.x += bandAmplitude / 100.0 * mod(warpAmount, 1.0) * mod(warpAmount, 1.0);

    // RGB to YUV
    vec2 leftUV = warpUV;
    leftUV.x -= chromaticAberration / 100.0;

    vec2 rightUV = warpUV;
    rightUV.x += chromaticAberration / 100.0;

    vec3 yuvLeft = RGB2YUV(texture2D(sc_ScreenTexture, leftUV).rgb);
    vec3 yuvCenter = RGB2YUV(texture2D(sc_ScreenTexture, warpUV).rgb);
    vec3 yuvRight = RGB2YUV(texture2D(sc_ScreenTexture, rightUV).rgb);

    vec3 rgbCenter = YUV2RGB(vec3(yuvCenter.x, baseSaturation * yuvCenter.y + yuvLeft.y, baseSaturation * yuvCenter.z + yuvRight.z));
    
    gl_FragColor = vec4(rgbCenter.r, rgbCenter.g, rgbCenter.b, 1.0);
}

#endif //FRAGMENT_SHADER
