varying vec2 texCoord;
varying vec2 maskTexCoord;
varying vec2 uv;

#ifdef VERTEX_SHADER

void main(void)
{
    gl_Position = vec4(atbCoord, 0.0, 1.0);
    uv = atbCoord * 0.5 + 0.5;
    texCoord = atbTexCoord;
    maskTexCoord = atbMaskTexCoord;
}

#endif


#ifdef FRAGMENT_SHADER

#define KERNEL_SIZE 9
// // Sharpen
// // -1 -1 -1
// // -1 +9 -1
// // -1 -1 -1

vec2 offset[KERNEL_SIZE];
float kernel[KERNEL_SIZE];

mat3 YUVFromRGB = mat3(
   vec3(0.299,-0.14713, 0.615),
   vec3(0.587,-0.28886,-0.51499),
   vec3(0.114,0.436,-0.10001));

mat3 RGBFromYUV = mat3(
    vec3(1.0, 1.0, 1.0),
    vec3(0.0,-0.394,2.03211),
    vec3(1.13983,-0.580,0.0));

float extractLuma(vec3 c)
{
    return c.r * 0.299 + c.g * 0.587 + c.b * 0.114;
}

void main(void)
{ 
    vec4 originalColor = texture2D(uniOriginalImage, uv);
    vec4 color = texture2D(uniTexture, texCoord);
    float maskAlpha = texture2D(uniMask, maskTexCoord).r;
   
    vec2 delta = vec2(1.0 / 720.0, 1.0 / 1280.0);
    
    offset[0] = vec2(-delta.x, -delta.y);
    offset[1] = vec2(0.0, -delta.y);
    offset[2] = vec2(delta.x, -delta.y);

    offset[3] = vec2(-delta.x, 0.0);
    offset[4] = vec2(0.0, 0.0);
    offset[5] = vec2(delta.x, 0.0);

    offset[6] = vec2(-delta.x, delta.y);
    offset[7] = vec2(0.0, delta.y);
    offset[8] = vec2(delta.x, delta.y);

    kernel[0] = -1.0;   kernel[1] = -1.0;   kernel[2] = -1.0;
    kernel[3] = -1.0;   kernel[4] = +9.0;   kernel[5] = -1.0;
    kernel[6] = -1.0;   kernel[7] = -1.0;   kernel[8] = -1.0;

    vec4 accum = vec4(0.0); 
    for(int i = 0; i < KERNEL_SIZE; i++) {
        vec4 tmp = texture2D(uniOriginalImage, uv + offset[i]);
        accum += tmp * kernel[i];
    }

    float finalAlpha = uniAlpha * maskAlpha;
    vec4 res = originalColor * (1.0 - finalAlpha) + accum * finalAlpha;
    
    gl_FragColor = res;    
}

#endif
