
varying vec2 texCoord;
varying vec2 maskTexCoord;
varying vec2 originalImageCoord;

#ifdef VERTEX_SHADER

void main(void)
{
    gl_Position = vec4(atbCoord, 0.0, 1.0);
    originalImageCoord = atbCoord * 0.5 + 0.5;
    texCoord = atbTexCoord;
    maskTexCoord = atbMaskTexCoord;
}

#endif


#ifdef FRAGMENT_SHADER

#include "shaders/common/definedBlendMode.glsl"


vec4 mapColor(vec4 originalColor)
{
    float EPS = 0.000001;
    float colorMapSubsampling = 5.0;
    vec4 fullColor = originalColor;
    vec3 pixValue = fullColor.rgb;
    float pxHeight = 512.0;
    float pxWidth = 512.0;
    float pxPerTable = 64.0;
    
    float bValue = (pixValue.b * 255.0) / 4.0;
    //    bValue = bValue + 0.5 / 4.0 - 0.5; //Center it
    float mulB = bValue;
    
    float mulB1 = clamp(floor(bValue), 0.0, 63.0);
    float row1 = floor(mulB1 / 8.0 + EPS);
    float col1 = floor(mulB1 - row1 * 8.0 + EPS);
    float lookupY1 = (pixValue.g * 63.0 + row1 * 64.0 + 0.5) / pxHeight;
    float lookupX1 = (pixValue.r * 63.0 + col1 * 64.0 + 0.5) / pxWidth;
    
    float mulB2 = clamp(floor(bValue + 1.0), 0.0, 63.0);
    float row2 = floor(mulB2 / 8.0 + EPS);
    float col2 = floor(mulB2 - row2 * 8.0 + EPS);
    float lookupY2 = (pixValue.g * 63.0 + row2 * 64.0 + 0.5) / pxHeight;
    float lookupX2 = (pixValue.r * 63.0 + col2 * 64.0 + 0.5) / pxWidth;
    
    float b1w = mulB - mulB1;

    vec3 sampled1 = texture2D(uniTexture, vec2(lookupX1, lookupY1)).rgb;
    vec3 sampled2 = texture2D(uniTexture, vec2(lookupX2, lookupY2)).rgb;
    
    vec3 res = mix(sampled1, sampled2, b1w);
    
    return vec4(res, fullColor.a);
}

void main(void)
{
#ifdef FETCH_FRAMEBUFFER
    vec4 originalColor = gl_LastFragData[0];
#else
    vec4 originalColor = texture2D(uniOriginalImage, originalImageCoord);
#endif
    vec4 color = mapColor(originalColor);
    color.a *= texture2D(uniMask, maskTexCoord).r;
    color.a *= uniAlpha;

    gl_FragColor = vec4((color * color.a + originalColor * (1.0 - color.a)).rgb, 1.0);
}

#endif
